@@ -1,4 +1,6 @@
 import math
+import os
+import sys
 from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
@@ -57,46 +59,46 @@ class AbundanceTable:
         """
         set the current table from AG89, WILM or ASPL
 
-        :param table: 
+        :param table:
         :type table: str
-        :returns: 
+        :returns:
 
         """
-        
-        
-        
+
         old_table = self._current_table
 
         self._current_table = table
-        
-        
+
         if self.current_table not in self.tables:
 
-            log.error(f"{self.name} does not contain {table} choose {','.join(self.table)}")
+            log.error(
+                f"{self.name} does not contain {table} choose {','.join(self.table)}"
+            )
 
             self._current_table = old_table
-            
+
             raise AssertionError()
-            
 
     @property
     def current_table(self) -> str:
 
-        convert= {
-        "AG89": "angr",
-        "ASPL": "aspl",
-        "WILM": "wilm"
-        }
+        convert = {"AG89": "angr", "ASPL": "aspl", "WILM": "wilm"}
 
         return convert[self._current_table]
-        
+
     @property
     def info(self) -> str:
 
         _abund_info = {}
-        _abund_info["WILM"] = "wilms\nfrom Wilms, Allen & McCray (2000), ApJ 542, 914 \n except for elements not listed which are given zero abundance)\n https://heasarc.nasa.gov/xanadu/xspec/manual/XSabund.html "
-        _abund_info["AG89"] = "angr\nfrom Anders E. & Grevesse N. (1989, Geochimica et Cosmochimica Acta 53, 197)\n https://heasarc.nasa.gov/xanadu/xspec/manual/XSabund.html"
-        _abund_info["ASPL"] = "aspl\nfrom Asplund M., Grevesse N., Sauval A.J. & Scott P. (2009, ARAA, 47, 481)\nhttps://heasarc.nasa.gov/xanadu/xspec/manual/XSabund.html"
+        _abund_info[
+            "WILM"
+        ] = "wilms\nfrom Wilms, Allen & McCray (2000), ApJ 542, 914 \n except for elements not listed which are given zero abundance)\n https://heasarc.nasa.gov/xanadu/xspec/manual/XSabund.html "
+        _abund_info[
+            "AG89"
+        ] = "angr\nfrom Anders E. & Grevesse N. (1989, Geochimica et Cosmochimica Acta 53, 197)\n https://heasarc.nasa.gov/xanadu/xspec/manual/XSabund.html"
+        _abund_info[
+            "ASPL"
+        ] = "aspl\nfrom Asplund M., Grevesse N., Sauval A.J. & Scott P. (2009, ARAA, 47, 481)\nhttps://heasarc.nasa.gov/xanadu/xspec/manual/XSabund.html"
 
         return _abund_info[self._current_table]
 
@@ -106,11 +108,10 @@ class AbundanceTable:
         """
         returns the XSECT table for the current model
 
-        :returns: 
+        :returns:
 
         """
-        _path: Path = Path(
-        "xsect") / f"xsect_{self.name}_{self.current_table}.fits"
+        _path: Path = Path("xsect") / f"xsect_{self.name}_{self.current_table}.fits"
 
         path_to_xsect: Path = _get_data_file_path(_path)
 
@@ -120,15 +121,14 @@ class AbundanceTable:
             xsect_ene = dxs["ENERGY"]
             xsect_val = dxs["SIGMA"]
 
-        return np.array(xsect_ene, dtype=np.float64), np.array(xsect_val,
-                                                           dtype=np.float64)
+        return np.array(xsect_ene, dtype=np.float64), np.array(
+            xsect_val, dtype=np.float64
+        )
 
 
-        
-phabs = AbundanceTable("phabs", ["angr","aspl"], "AG89")
-tbabs = AbundanceTable("tbabs", ["angr","aspl","wilm"], "WILM")
+phabs = AbundanceTable("phabs", ["angr", "aspl"], "AG89")
+tbabs = AbundanceTable("tbabs", ["angr", "aspl", "wilm"], "WILM")
 wabs = AbundanceTable("wabs", ["angr"], "AG89")
-    
 
 
 # PhAbs class
@@ -158,47 +158,50 @@ class PhAbs(Function1D, metaclass=FunctionMeta):
             delta : 0.1
             fix: True
 
+    properties:
+         abundance_table:
+            desc: the abundance table for the model
+            initial value: AG89
+            allowed values:
+            - AG89
+            - ASPL
+            function: _init_xsect
+
     """
+
     def _setup(self):
-        self._fixed_units = (astropy_units.keV,
-                             astropy_units.dimensionless_unscaled)
-        self.init_xsect(astromodels_config.absorption_models.phabs_table.value)
+        self._fixed_units = (
+            astropy_units.keV,
+            astropy_units.dimensionless_unscaled,
+        )
+
+        # # astromodels_config.absorption_models.phabs_table.value
+        # self.init_xsect(self.abundance_table.value)
 
     def _set_units(self, x_unit, y_unit):
-        self.NH.unit = astropy_units.cm**(-2)
+        self.NH.unit = astropy_units.cm ** (-2)
         self.redshift.unit = astropy_units.dimensionless_unscaled
 
-    def init_xsect(self, abund_table="AG89"):
+    def _init_xsect(self):
         """
         Set the abundance table
 
-        :param abund_table: "ASPL", "AG89" 
-        :returns: 
-        :rtype: 
+        :param abund_table: "ASPL", "AG89"
+        :returns:
+        :rtype:
 
         """
 
         # load cross section data
 
-        try:
-            phabs.set_table(abund_table)
-
-            log.debug(f"phabs model set to {abund_table}")
-            
-        except:
-
-            log.info(f"defaulting to {astromodels_config.absorption_models.phabs_table.value}")
-
-            phabs.set_table(astromodels_config.absorption_models.phabs_table.value)
-            
+        phabs.set_table(self.abundance_table.value)
 
         self.xsect_ene, self.xsect_val = phabs.xsect_table
 
     @property
-    def abundance_table(self):
+    def abundance_table_info(self):
         print(phabs.info)
 
-        
     def evaluate(self, x, NH, redshift):
 
         if isinstance(x, astropy_units.Quantity):
@@ -214,13 +217,12 @@ class PhAbs(Function1D, metaclass=FunctionMeta):
             _redshift = redshift
             _x = x
 
-        xsect_interp = interp(self.xsect_ene, self.xsect_val,
-                              _x * (1 + _redshift))
+        xsect_interp = interp(self.xsect_ene, self.xsect_val, _x * (1 + _redshift))
 
         # evaluate the exponential with numba
-        
+
         spec = _numba_eval(NH, xsect_interp) * _y_unit
-        
+
         return spec
 
 
@@ -251,45 +253,52 @@ class TbAbs(Function1D, metaclass=FunctionMeta):
             delta : 0.1
             fix: True
 
+    properties:
+         abundance_table:
+            desc: the abundance table for the model
+            initial value: WILM
+            allowed values:
+            - WILM
+            - AG89
+            - ASPL
+            function: _init_xsect
+
+
+
     """
+
     def _setup(self):
 
-        self.init_xsect(astromodels_config.absorption_models.tbabs_table.value)
+        # self.init_xsect(self.abundance_table)
 
-        self._fixed_units = (astropy_units.keV,
-                             astropy_units.dimensionless_unscaled)
+        self._fixed_units = (
+            astropy_units.keV,
+            astropy_units.dimensionless_unscaled,
+        )
 
     def _set_units(self, x_unit, y_unit):
 
-        self.NH.unit = astropy_units.cm**(-2)
+        self.NH.unit = astropy_units.cm ** (-2)
         self.redshift.unit = astropy_units.dimensionless_unscaled
 
-    def init_xsect(self, abund_table="WILM"):
+    def _init_xsect(self):
         """
         Set the abundance table
 
-        :param abund_table: "WILM", "ASPL", "AG89" 
-        :returns: 
-        :rtype: 
+        :param abund_table: "WILM", "ASPL", "AG89"
+        :returns:
+        :rtype:
 
         """
-        
-        try:
-            tbabs.set_table(abund_table)
-            
-            log.debug(f"tbabs model set to {abund_table}")
-            
-        except:
 
-            log.info(f"defaulting to {astromodels_config.absorption_models.tbabs_table.value}")
-
-            tbabs.set_table(astromodels_config.absorption_models.tbabs_table.value)
-            
+        tbabs.set_table(self.abundance_table.value)
 
         self.xsect_ene, self.xsect_val = tbabs.xsect_table
 
+        log.debug(f"updated the TbAbs table to {self.abundance_table.value}")
+
     @property
-    def abundance_table(self):
+    def abundance_table_info(self):
         print(tbabs.info)
 
     def evaluate(self, x, NH, redshift):
@@ -300,7 +309,6 @@ class TbAbs(Function1D, metaclass=FunctionMeta):
             _y_unit = astropy_units.dimensionless_unscaled
             _x = x.value
             _redshift = redshift.value
-
         else:
 
             _unit = 1.0
@@ -308,8 +316,7 @@ class TbAbs(Function1D, metaclass=FunctionMeta):
             _redshift = redshift
             _x = x
 
-        xsect_interp = interp(self.xsect_ene, self.xsect_val,
-                              _x * (1 + _redshift))
+        xsect_interp = interp(self.xsect_ene, self.xsect_val, _x * (1 + _redshift))
 
         spec = _numba_eval(NH, xsect_interp) * _y_unit
 
@@ -345,31 +352,31 @@ class WAbs(Function1D, metaclass=FunctionMeta):
     """
 
     def _setup(self):
-        self._fixed_units = (astropy_units.keV,
-                             astropy_units.dimensionless_unscaled)
-        self.init_xsect()
+        self._fixed_units = (
+            astropy_units.keV,
+            astropy_units.dimensionless_unscaled,
+        )
+        self._init_xsect()
 
     def _set_units(self, x_unit, y_unit):
-        self.NH.unit = astropy_units.cm**(-2)
+        self.NH.unit = astropy_units.cm ** (-2)
         self.redshift.unit = astropy_units.dimensionless_unscaled
 
-    def init_xsect(self):
+    def _init_xsect(self):
         """
         Set the abundance table
 
         :returns:
         :rtype:
 
-        """            
+        """
 
         self.xsect_ene, self.xsect_val = wabs.xsect_table
 
-
     @property
-    def abundance_table(self):
+    def abundance_table_info(self):
         print(wabs.info)
 
-
     def evaluate(self, x, NH, redshift):
 
         if isinstance(x, astropy_units.Quantity):
@@ -385,17 +392,13 @@ class WAbs(Function1D, metaclass=FunctionMeta):
             _redshift = redshift
             _x = x
 
-        xsect_interp = interp(self.xsect_ene, self.xsect_val,
-                              _x * (1 + _redshift))
+        xsect_interp = interp(self.xsect_ene, self.xsect_val, _x * (1 + _redshift))
 
-        xsect_interp = interp( self.xsect_ene, self.xsect_val, _x * (1+ _redshift))
-
-        spec = _numba_eval(NH,xsect_interp) * _y_unit
+        spec = _numba_eval(NH, xsect_interp) * _y_unit
 
         return spec
 
 
-
 if has_ebltable:
 
     class EBLattenuation(Function1D, metaclass=FunctionMeta):
@@ -421,17 +424,30 @@ if has_ebltable:
                 max : 10.0
                 fix : yes
 
+        properties:
+           ebl_model:
+              desc: set the EBL model
+              initial value: dominguez
+              allowed values:
+                 - dominguez
+                 - franceschini
+                 - kneiske
+                 - inuoe
+                 - gilmore
+              function: _set_ebl_model
+
+
         """
 
-        def _setup(self):
+        # def _setup(self):
 
-            # define EBL model, use dominguez as default
-            self._tau = ebltau.OptDepth.readmodel(model=astromodels_config.absorption_models.ebl_table.value)
+        #     # define EBL model, use dominguez as default
+        #     self._tau = ebltau.OptDepth.readmodel(model=astromodels_config.absorption_models.ebl_table.value)
 
-        def set_ebl_model(self, modelname):
+        def _set_ebl_model(self):
 
             # passing modelname to ebltable, which will check if defined
-            self._tau = ebltau.OptDepth.readmodel(model=modelname)
+            self._tau = ebltau.OptDepth.readmodel(model=self.ebl_model.value)
 
         def _set_units(self, x_unit, y_unit):
 
@@ -449,8 +465,7 @@ if has_ebltable:
                 not hasattr(y_unit, "physical_type")
                 or y_unit.physical_type != "dimensionless"
             ):
-                raise InvalidUsageForFunction(
-                    "Unit for y is not dimensionless.")
+                raise InvalidUsageForFunction("Unit for y is not dimensionless.")
 
             self.redshift.unit = astropy_units.dimensionless_unscaled
             self.attenuation.unit = astropy_units.dimensionless_unscaled
@@ -468,22 +483,23 @@ if has_ebltable:
             else:
 
                 # otherwise it's in keV
-                eTeV = x/1.e9
+                eTeV = x / 1.0e9
 
-                _unit = 1.
+                _unit = 1.0
                 _redshift = redshift
                 _attenuation = attenuation
-                
-                
-            return _numba_eval(self._tau.opt_depth(_redshift, eTeV), _attenuation) * _unit
 
+            return (
+                _numba_eval(self._tau.opt_depth(_redshift, eTeV), _attenuation) * _unit
+            )
 
 
 @nb.vectorize
 def _exp(x):
     return math.exp(x)
 
+
 @nb.njit(fastmath=True)
 def _numba_eval(nh, xsect_interp):
 
-    return _exp(-nh * xsect_interp )
+    return _exp(-nh * xsect_interp)
