@@ -4,16 +4,21 @@ import math
 
 import astropy.units as astropy_units
 import numpy as np
-from scipy.special import  erfcinv, erf
+from scipy.special import erfcinv, erf
 
-from astromodels.functions.function import Function1D, FunctionMeta, ModelAssertionViolation
+from astromodels.functions.function import (
+    Function1D,
+    FunctionMeta,
+    ModelAssertionViolation,
+)
 
 
-
-deg2rad = old_div(np.pi,180.)
-rad2deg = old_div(180.,np.pi)
+deg2rad = old_div(np.pi, 180.0)
+rad2deg = old_div(180.0, np.pi)
 
 # noinspection PyPep8Naming
+
+
 class Gaussian(Function1D, metaclass=FunctionMeta):
     r"""
     description :
@@ -71,7 +76,11 @@ class Gaussian(Function1D, metaclass=FunctionMeta):
 
         norm = old_div(self.__norm_const, sigma)
 
-        return F * norm * np.exp(old_div(-np.power(x - mu, 2.), (2 * np.power(sigma, 2.))))
+        return (
+            F
+            * norm
+            * np.exp(old_div(-np.power(x - mu, 2.0), (2 * np.power(sigma, 2.0))))
+        )
 
     def from_unit_cube(self, x):
         """
@@ -98,6 +107,7 @@ class Gaussian(Function1D, metaclass=FunctionMeta):
 
         return res
 
+
 class Truncated_gaussian(Function1D, metaclass=FunctionMeta):
     r"""
     description :
@@ -167,17 +177,15 @@ class Truncated_gaussian(Function1D, metaclass=FunctionMeta):
         # sigma has the same dimensions as x
         self.sigma.unit = x_unit
 
-
     # noinspection PyPep8Naming
     def evaluate(self, x, F, mu, sigma, lower_bound, upper_bound):
 
-
         # phi is in unitless, so we need to do this trick
         # to keep the units right
 
         norm = old_div(self.__norm_const, sigma)
 
-        phi = np.zeros(x.shape) * F * norm * 0.
+        phi = np.zeros(x.shape) * F * norm * 0.0
         idx = (x >= lower_bound) & (x <= upper_bound)
 
         sqrt_two = 1.414213562
@@ -187,15 +195,16 @@ class Truncated_gaussian(Function1D, metaclass=FunctionMeta):
         lower_arg = old_div((lower_bound - mu), sigma)
         upper_arg = old_div((upper_bound - mu), sigma)
 
-
-
         # the typical gaussian functions
 
-        phi[idx] = np.exp(old_div(-np.power(x[idx] - mu, 2.), (2 * np.power(sigma, 2.)))) * F * norm
+        phi[idx] = (
+            np.exp(old_div(-np.power(x[idx] - mu, 2.0), (2 * np.power(sigma, 2.0))))
+            * F
+            * norm
+        )
 
         # the denominator is a function of the CDF
 
-
         if isinstance(F, astropy_units.Quantity):
             # erf cannot accept units
 
@@ -206,9 +215,6 @@ class Truncated_gaussian(Function1D, metaclass=FunctionMeta):
 
         theta_upper = 0.5 + 0.5 * erf(old_div(upper_arg, sqrt_two))
 
-
-
-
         return old_div(phi, (theta_upper - theta_lower))
 
     def from_unit_cube(self, x):
@@ -237,9 +243,10 @@ class Truncated_gaussian(Function1D, metaclass=FunctionMeta):
         arg = theta_lower + x * (theta_upper - theta_lower)
 
         out = mu + sigma * sqrt_two * erfcinv(2 * (1 - arg))
-        
+
         return np.clip(out, lower_bound, upper_bound)
 
+
 class Cauchy(Function1D, metaclass=FunctionMeta):
     r"""
     description :
@@ -352,16 +359,17 @@ class Cosine_Prior(Function1D, metaclass=FunctionMeta):
 
     """
 
-
     def _setup(self):
 
-        self._fixed_units = (astropy_units.dimensionless_unscaled,astropy_units.dimensionless_unscaled)
+        self._fixed_units = (
+            astropy_units.dimensionless_unscaled,
+            astropy_units.dimensionless_unscaled,
+        )
 
         self._is_prior = True
 
     def _set_units(self, x_unit, y_unit):
 
-
         # this prior needs to use the fixed units and
         # they do not need to be converted as they have no
         # dimension
@@ -381,21 +389,21 @@ class Cosine_Prior(Function1D, metaclass=FunctionMeta):
 
         return True
 
-    def evaluate(self, x, lower_bound, upper_bound,value):
+    def evaluate(self, x, lower_bound, upper_bound, value):
         # The value * 0 is to keep the units right
 
         result = np.zeros(x.shape) * value * 0
 
         idx = (x >= lower_bound) & (x <= upper_bound)
 
-        norm = (np.sin(deg2rad*(upper_bound)) - np.sin(deg2rad*(lower_bound))) * 57.29577795
+        norm = (
+            np.sin(deg2rad * (upper_bound)) - np.sin(deg2rad * (lower_bound))
+        ) * 57.29577795
 
-
-        result[idx] = value * np.cos(deg2rad*( x[idx] )) / norm
+        result[idx] = value * np.cos(deg2rad * (x[idx])) / norm
 
         return result
 
-
     def from_unit_cube(self, x):
         """
         Used by multinest
@@ -405,8 +413,8 @@ class Cosine_Prior(Function1D, metaclass=FunctionMeta):
         :param upper_bound:
         :return:
         """
-        cosdec_min = np.cos(deg2rad*(90.0 + self.lower_bound.value))
-        cosdec_max = np.cos(deg2rad*(90.0 + self.upper_bound.value))
+        cosdec_min = np.cos(deg2rad * (90.0 + self.lower_bound.value))
+        cosdec_max = np.cos(deg2rad * (90.0 + self.upper_bound.value))
 
         v = x * (cosdec_max - cosdec_min)
         v += cosdec_min
@@ -425,39 +433,39 @@ class Cosine_Prior(Function1D, metaclass=FunctionMeta):
 
 class Log_normal(Function1D, metaclass=FunctionMeta):
     r"""
-       description :
+    description :
 
-           A log normal function
+        A log normal function
 
-       latex : $ K \frac{1}{ x \sigma \sqrt{2 \pi}}\exp{\frac{(\log x/piv - \mu/piv)^2}{2~(\sigma)^2}} $
+    latex : $ K \frac{1}{ x \sigma \sqrt{2 \pi}}\exp{\frac{(\log x/piv - \mu/piv)^2}{2~(\sigma)^2}} $
 
-       parameters :
+    parameters :
 
-           F :
-               desc : Integral between 0and +inf. Fix this to 1 to obtain a log Normal distribution
-               initial value : 1
+        F :
+            desc : Integral between 0and +inf. Fix this to 1 to obtain a log Normal distribution
+            initial value : 1
 
-           mu :
+        mu :
 
-               desc : Central value
-               initial value : 0.0
+            desc : Central value
+            initial value : 0.0
 
-           sigma :
+        sigma :
 
-               desc : standard deviation
-               initial value : 1.0
-               min : 1e-12
+            desc : standard deviation
+            initial value : 1.0
+            min : 1e-12
 
-           piv :
-               desc : pivot. Leave this to 1 for a proper log normal distribution
-               initial value : 1.0
-               fix : yes
+        piv :
+            desc : pivot. Leave this to 1 for a proper log normal distribution
+            initial value : 1.0
+            fix : yes
 
-       tests :
-           - { x : 0.0, function value: 0.3989422804014327, tolerance: 1e-10}
-           - { x : -1.0, function value: 0.24197072451914337, tolerance: 1e-9}
+    tests :
+        - { x : 0.0, function value: 0.3989422804014327, tolerance: 1e-10}
+        - { x : -1.0, function value: 0.24197072451914337, tolerance: 1e-9}
 
-       """
+    """
 
     # Place this here to avoid recomputing it all the time
 
@@ -491,10 +499,19 @@ class Log_normal(Function1D, metaclass=FunctionMeta):
 
         # The log normal is not defined if x < 0. The "0 * x" part is to conserve the units if
         # x has them, because 0 * x will be a Quantity with the same units as x
-        idx = (x > 0 * x)
+        idx = x > 0 * x
 
-        result[idx] = F * self.__norm_const / (sigma / piv * x / piv) * np.exp(
-            old_div(-np.power(np.log(old_div(x, piv)) - old_div(mu, piv), 2.), (2 * np.power(old_div(sigma, piv), 2.))))
+        result[idx] = (
+            F
+            * self.__norm_const
+            / (sigma / piv * x[idx] / piv)
+            * np.exp(
+                old_div(
+                    -np.power(np.log(old_div(x[idx], piv)) - old_div(mu, piv), 2.0),
+                    (2 * np.power(old_div(sigma, piv), 2.0)),
+                )
+            )
+        )
 
         return result
 
@@ -581,7 +598,6 @@ class Uniform_prior(Function1D, metaclass=FunctionMeta):
 
         return result
 
-
     def from_unit_cube(self, x):
         """
         Used by multinest
@@ -602,6 +618,7 @@ class Uniform_prior(Function1D, metaclass=FunctionMeta):
 
         return par
 
+
 class Log_uniform_prior(Function1D, metaclass=FunctionMeta):
     r"""
     description :
