@@ -1,10 +1,7 @@
-from __future__ import division
-
 import collections
 import gc
 import os
 import re
-from builtins import object, range, str
 from dataclasses import dataclass
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Union
@@ -14,14 +11,13 @@ import astropy.units as u
 import h5py
 import numpy as np
 import scipy.interpolate
-from future.utils import with_metaclass
-from interpolation import interp
-from interpolation.splines import eval_linear
-
 from astromodels.core.parameter import Parameter
 from astromodels.functions.function import Function1D, FunctionMeta
 from astromodels.utils import get_user_data_path
 from astromodels.utils.logging import setup_logger
+from future.utils import with_metaclass
+from interpolation import interp
+from interpolation.splines import eval_linear
 
 log = setup_logger(__name__)
 
@@ -78,29 +74,30 @@ class UnivariateSpline(object):
 class TemplateModelFactory(object):
     def __init__(
         self,
-        name,
-        description,
-        energies,
-        names_of_parameters,
-        interpolation_degree=1,
-        spline_smoothing_factor=0,
+        name: str,
+        description: str,
+        energies: np.ndarray,
+        names_of_parameters: List[str],
+        interpolation_degree: int = 1,
+        spline_smoothing_factor: int = 0,
     ):
 
         # Store model name
 
         # Enforce that it does not contain spaces nor strange characters
 
-        name = str(name)
+        name: str = str(name)
 
         if re.match("[a-zA-Z_][a-zA-Z0-9_]*", name) is None:
             log.error(
                 "The provided name '%s' is not a valid name. You cannot use spaces, "
-                "or special characters")
+                "or special characters"
+            )
             raise RuntimeError()
 
-        self._name = name
+        self._name: str = name
 
-        self._description = str(description)
+        self._description: str = str(description)
 
         # Store energy grid
 
@@ -110,16 +107,18 @@ class TemplateModelFactory(object):
                 "Energy unit is not a Quantity instance, so units has not been provided. Using keV."
             )
 
-            energies = energies * u.keV
+            energies: u.Quantity = energies * u.keV
 
-        self._energies = np.array(energies.to(u.keV).value)
+        self._energies: np.ndarray = np.array(energies.to(u.keV).value)
 
         # Enforce that they are ordered
         self._energies.sort()
 
         # We create a dictionary which will contain the grid for each parameter
 
-        self._parameters_grids = collections.OrderedDict()
+        self._parameters_grids: Dict[
+            str, Optional[np.ndarray]
+        ] = collections.OrderedDict()
 
         for parameter_name in names_of_parameters:
 
@@ -129,18 +128,19 @@ class TemplateModelFactory(object):
 
         self._interpolators = None
 
-        self._interpolation_degree = interpolation_degree
+        self._interpolation_degree: int = interpolation_degree
 
-        self._spline_smoothing_factor = int(spline_smoothing_factor)
+        self._spline_smoothing_factor: int = int(spline_smoothing_factor)
 
-    def define_parameter_grid(self, parameter_name: str,
-                              grid: np.ndarray) -> None:
+    def define_parameter_grid(
+        self, parameter_name: str, grid: np.ndarray
+    ) -> None:
         """
         Define the parameter grid for this parameter.
         Pass the name of the parameter and the array of values that it will take in the grid
         """
 
-        if not parameter_name in self._parameters_grids:
+        if parameter_name not in self._parameters_grids:
 
             log.error(f"Parameter {parameter_name} is not part of this model")
 
@@ -151,7 +151,8 @@ class TemplateModelFactory(object):
         if not (grid_.shape[0] > 1):
 
             log.error(
-                "A grid for a parameter must contain at least two elements")
+                "A grid for a parameter must contain at least two elements"
+            )
 
             raise AssertionError()
 
@@ -160,14 +161,18 @@ class TemplateModelFactory(object):
         if not np.all(np.unique(grid_) == grid_):
 
             log.error(
-                f"Non-unique elements in grid for parameter {parameter_name}")
+                f"Non-unique elements in grid for parameter {parameter_name}"
+            )
 
             raise AssertionError()
 
         self._parameters_grids[parameter_name] = grid_
 
-    def add_interpolation_data(self, differential_fluxes,
-                               **parameters_values_input):
+    def add_interpolation_data(
+        self,
+        differential_fluxes: np.ndarray,
+        **parameters_values_input: Dict[str, float],
+    ):
 
         # Verify that the grid has been defined for all parameters
 
@@ -177,7 +182,8 @@ class TemplateModelFactory(object):
 
                 log.error(
                     "You need to define a grid for all parameters, by using the "
-                    "define_parameter_grid method.")
+                    "define_parameter_grid method."
+                )
 
                 raise IncompleteGrid()
 
@@ -221,14 +227,15 @@ class TemplateModelFactory(object):
 
         for i, (k, v) in enumerate(self._parameters_grids.items()):
 
-            if not k in parameters_values_input:
+            if k not in parameters_values_input:
 
                 log.error(f"Parameter {k} is not in input")
 
                 raise AssertionError()
 
             parameter_idx.append(
-                int(np.where(v == parameters_values_input[k])[0][0]))
+                int(np.where(v == parameters_values_input[k])[0][0])
+            )
 
         log.debug(f" have index {parameter_idx}")
 
@@ -245,20 +252,24 @@ class TemplateModelFactory(object):
 
         if not isinstance(differential_fluxes, u.Quantity):
 
-            differential_fluxes = (np.array(differential_fluxes) * 1 /
-                                   (u.keV * u.s * u.cm**2))  # type: u.Quantity
+            differential_fluxes = (
+                np.array(differential_fluxes) * 1 / (u.keV * u.s * u.cm ** 2)
+            )  # type: u.Quantity
 
         # Then we transform it in the right units and we cast it back to a pure np.array
 
         differential_fluxes = np.array(
-            differential_fluxes.to(1/ (u.keV * u.s * u.cm**2)).value)
+            differential_fluxes.to(1 / (u.keV * u.s * u.cm ** 2)).value
+        )
 
         # Now let's check for valid inputs
 
         if not self._energies.shape[0] == differential_fluxes.shape[0]:
 
-            log.error("Differential fluxes and energies must have "
-                      "the same number of elements")
+            log.error(
+                "Differential fluxes and energies must have "
+                "the same number of elements"
+            )
 
             raise AssertionError()
 
@@ -266,9 +277,7 @@ class TemplateModelFactory(object):
         # log space)
         if not np.all(np.isfinite(differential_fluxes)):
 
-            log.error(
-                "You have invalid values in the differential flux (nan or inf)"
-            )
+            log.error("You have invalid values in the differential flux (nan or inf)")
 
             raise AssertionError()
 
@@ -276,7 +285,8 @@ class TemplateModelFactory(object):
 
             log.error(
                 "You have negative values in the differential flux (which is of "
-                "course impossible)")
+                "course impossible)"
+            )
 
             raise AssertionError()
 
@@ -284,8 +294,9 @@ class TemplateModelFactory(object):
 
             log.warning(
                 "You have zeros in the differential flux. Since the interpolation happens in the log space, "
-                "this cannot be accepted. We will substitute zeros with %g" %
-                _TINY_)
+                "this cannot be accepted. We will substitute zeros with %g"
+                % _TINY_
+            )
 
             idx = differential_fluxes == 0  # type: np.ndarray
             differential_fluxes[idx] = _TINY_
@@ -295,7 +306,8 @@ class TemplateModelFactory(object):
         # Now set the values in the data frame
 
         self._data_frame[tuple(parameter_idx)] = np.atleast_2d(
-            differential_fluxes)
+            differential_fluxes
+        )
 
     def save_data(self, overwrite: bool = False):
 
@@ -303,9 +315,13 @@ class TemplateModelFactory(object):
 
         if np.any(np.isnan(self._data_frame)):
 
-            log.error("You have NaNs in the data matrix. Usually this means "
-                      "that you didn't fill it up completely, or that some of "
-                      "your data contains nans. Cannot save the file.")
+            log.error(
+                "You have NaNs in the data matrix. Usually this means "
+                "that you didn't fill it up completely, or that some of "
+                "your data contains nans. Cannot save the file."
+            )
+
+            raise AssertionError()
 
             raise AssertionError()
 
@@ -315,10 +331,9 @@ class TemplateModelFactory(object):
 
         # Sanitize the data file
 
-        filename_sanitized: Path = data_dir_path / f"{self._name}.h5"
+        filename_sanitized: Path = data_dir_path.absolute() / f"{self._name}.h5"
 
         # Check that it does not exists
-
         if filename_sanitized.exists():
 
             if overwrite:
@@ -327,21 +342,29 @@ class TemplateModelFactory(object):
 
                     os.remove(filename_sanitized)
 
-                except:
+                except IOError:
 
                     log.error(
                         "The file %s already exists and cannot be removed (maybe you do not have "
-                        "permissions to do so?). " % filename_sanitized)
+                        "permissions to do so?). " % filename_sanitized
+                    )
 
-                    raise IOError()
+                    raise IOError(
+                        f"The file {filename_sanitized} already exists and cannot be removed (maybe you do not have "
+                        "permissions to do so?). "
+                    )
 
             else:
 
                 log.error(
                     "The file %s already exists! You cannot call two different "
-                    "template models with the same name" % filename_sanitized)
+                    "template models with the same name" % filename_sanitized
+                )
 
-                raise IOError()
+                raise IOError(
+                    f"The file {filename_sanitized} already exists! You cannot call two different "
+                    "template models with the same name"
+                )
 
         # Open the HDF5 file and write objects
 
@@ -353,11 +376,11 @@ class TemplateModelFactory(object):
             grid=self._data_frame,
             energies=self._energies,
             parameters=self._parameters_grids,
-            parameter_order=list(self._parameters_grids.keys()))
+            parameter_order=list(self._parameters_grids.keys()),
+        )
 
         template_file.save(filename_sanitized)
 
-   
 
 # This adds a method to a class at runtime
 
@@ -377,12 +400,14 @@ class RectBivariateSplineWrapper(object):
     syntax as the other interpolation methods
 
     """
+
     def __init__(self, *args, **kwargs):
 
         # We can use interp2, which features spline interpolation instead of linear interpolation
 
         self._interpolator = scipy.interpolate.RectBivariateSpline(
-            *args, **kwargs)
+            *args, **kwargs
+        )
 
     def __call__(self, x):
 
@@ -394,10 +419,11 @@ class RectBivariateSplineWrapper(object):
 @dataclass
 class TemplateFile:
     """
-    simple container to read and write 
+    simple container to read and write
     the data to an hdf5 file
 
     """
+
     name: str
     description: str
     grid: np.ndarray
@@ -407,15 +433,14 @@ class TemplateFile:
     interpolation_degree: int
     spline_smoothing_factor: float
 
-
     def save(self, file_name: str):
 
         """
         serialize the contents to a file
 
-        :param file_name: 
+        :param file_name:
         :type file_name: str
-        :returns: 
+        :returns:
 
         """
         with h5py.File(file_name, "w") as f:
@@ -431,12 +456,14 @@ class TemplateFile:
 
             # store the parameter order
             dt = h5py.special_dtype(vlen=str)
-            po = np.array(self.parameter_order, dtype=dt) 
-            f.create_dataset('parameter_order', data=po)
+            po = np.array(self.parameter_order, dtype=dt)
+            f.create_dataset("parameter_order", data=po)
             par_group = f.create_group("parameters")
             for k in self.parameter_order:
 
-                par_group.create_dataset(k, data=self.parameters[k], compression="gzip")
+                par_group.create_dataset(
+                    k, data=self.parameters[k], compression="gzip"
+                )
 
     @classmethod
     def from_file(cls, file_name: str):
@@ -444,20 +471,20 @@ class TemplateFile:
         """
         read contents from a file
 
-        :param cls: 
-        :type cls: 
-        :param file_name: 
+        :param cls:
+        :type cls:
+        :param file_name:
         :type file_name: str
-        :returns: 
+        :returns:
 
         """
-        
+
         with h5py.File(file_name, "r") as f:
 
             name = f.attrs["name"]
-            description = f.attrs["description"] 
-            interpolation_degree = f.attrs["interpolation_degree"] 
-            spline_smoothing_factor = f.attrs["spline_smoothing_factor"] 
+            description = f.attrs["description"]
+            interpolation_degree = f.attrs["interpolation_degree"]
+            spline_smoothing_factor = f.attrs["spline_smoothing_factor"]
 
             energies = f["energies"][()]
             parameter_order = f["parameter_order"][()]
@@ -469,30 +496,26 @@ class TemplateFile:
             for k in parameter_order:
 
                 parameters[k] = f["parameters"][k][()]
-            
+
+        return cls(
+            name=name,
+            description=description,
+            interpolation_degree=interpolation_degree,
+            spline_smoothing_factor=spline_smoothing_factor,
+            energies=energies,
+            parameter_order=parameter_order,
+            parameters=parameters,
+            grid=grid,
+        )
 
 
-        return cls(name=name,
-                   description=description,
-                   interpolation_degree=interpolation_degree,
-                   spline_smoothing_factor=spline_smoothing_factor,
-                   energies=energies,
-                   parameter_order=parameter_order,
-                   parameters = parameters,
-                   grid=grid
-                   )
-
-                
-        
-
-
-    
 class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
     r"""
     description :
         A template model
-    latex : $n.a.$
+    latex :
+        $n.a.$
     parameters :
         K :
             desc : Normalization (freeze this to 1 if the template provides the normalization by itself)
@@ -506,7 +529,12 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
             min : 1e-5
     """
 
-    def _custom_init_(self, model_name, other_name=None, log_interp=True):
+    def _custom_init_(
+        self,
+        model_name: str,
+        other_name: Optional[str] = None,
+        log_interp: bool = True,
+    ):
         """
         Custom initialization for this model
 
@@ -522,50 +550,47 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         # Sanitize the data file
 
-        filename_sanitized: Path = data_dir_path.absolute() /  f"{model_name}.h5"
-        
+        filename_sanitized = data_dir_path.absolute() / f"{model_name}.h5"
+
         if not filename_sanitized.exists():
 
-            log.error("The data file %s does not exists. Did you use the "
-                "TemplateFactory?" % (filename_sanitized))
-            
+            log.error(
+                f"The data file {filename_sanitized} does not exists. Did you use the "
+                "TemplateFactory?"
+            )
+
             raise MissingDataFile()
 
         # Open the template definition and read from it
 
-        self._data_file = filename_sanitized
+        self._data_file: Path = filename_sanitized
 
         # use the file shadow to read
-        
+
         template_file: TemplateFile = TemplateFile.from_file(filename_sanitized)
 
         self._parameters_grids = collections.OrderedDict()
 
-        
-
         for key in template_file.parameter_order:
 
             try:
 
                 # sometimes this is
                 # stored binary
-                
+
                 k = key.decode()
 
-            except(AttributeError):
+            except (AttributeError):
 
                 # if not, then we
                 # load as a normal str
-                
+
                 k = key
-                                
+
             log.debug(f"reading parameter {str(k)}")
 
-  
             self._parameters_grids[str(k)] = template_file.parameters[key]
 
-            
-
         self._energies = template_file.energies
 
         # Now get the metadata
@@ -585,7 +610,6 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         function_definition["latex"] = "n.a."
 
-        
         # Now build the parameters according to the content of the parameter grid
 
         parameters = collections.OrderedDict()
@@ -606,7 +630,9 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         if other_name is None:
 
-            super(TemplateModel, self).__init__(name, function_definition, parameters)
+            super(TemplateModel, self).__init__(
+                name, function_definition, parameters
+            )
 
         else:
 
@@ -619,11 +645,11 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
         self._prepare_interpolators(log_interp, template_file.grid)
 
         # try can clean up the file
-        
+
         del template_file
 
         gc.collect()
-                
+
     def _prepare_interpolators(self, log_interp, data_frame):
 
         # Figure out the shape of the data matrices
@@ -640,7 +666,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
             if log_interp:
 
                 this_data = np.array(
-                    np.log10(data_frame[...,i]).reshape(*data_shape),
+                    np.log10(data_frame[..., i]).reshape(*data_shape),
                     dtype=float,
                 )
 
@@ -655,7 +681,6 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
                 self._is_log10 = False
 
-                
             if len(list(self._parameters_grids.values())) == 2:
 
                 x, y = list(self._parameters_grids.values())
@@ -709,7 +734,12 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
                 # In more than 2d we can only use linear interpolation
 
                 this_interpolator = GridInterpolate(
-                    tuple([np.array(x) for x in list(self._parameters_grids.values())]),
+                    tuple(
+                        [
+                            np.array(x, dtype="<f8")
+                            for x in list(self._parameters_grids.values())
+                        ]
+                    ),
                     this_data,
                 )
 
@@ -717,7 +747,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         # clear the data
         self._data_frame = None
-            
+
     def _set_units(self, x_unit, y_unit):
 
         self.K.unit = y_unit
@@ -745,7 +775,7 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
             # Same for the scale
 
-            scale = scale.to(1/ u.keV).value
+            scale = scale.to(1 / u.keV).value
 
         if self._is_log10:
 
@@ -774,9 +804,11 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
         if self._is_log10:
 
-            interpolator = UnivariateSpline(np.log10(e_tilde), log_interpolations)
+            interpolator = UnivariateSpline(
+                np.log10(e_tilde), log_interpolations
+            )
 
-            values = np.power(10, interpolator(log_energies))
+            values = np.power(10.0, interpolator(log_energies))
 
         else:
 
@@ -795,25 +827,26 @@ class TemplateModel(with_metaclass(FunctionMeta, Function1D)):
 
     def clean(self):
         """
-        Table models can consume a lot of memory. If are creating lots of 
+        Table models can consume a lot of memory. If are creating lots of
         table models in memory for simulations, you may want to call
         clean on the model try and remove some of the memory consumed by the models
 
-        :returns: 
+        :returns:
 
         """
-        
+
         self._interpolators = None
         del self._interpolators
         gc.collect()
 
-        log.info("You have 'cleaned' the table model at it will no longer be useable")
+        log.info(
+            "You have 'cleaned' the table model and it will no longer be useable"
+        )
 
-    def __del__(self):
+    # def __del__(self):
 
-        self.clean()
+    #     self.clean()
 
-        
     @property
     def data_file(self):
 
@@ -875,6 +908,17 @@ class XSPECTableModel(object):
             energies = f["ENERGIES"]
             ene_lo = energies.data["ENERG_LO"]
             ene_hi = energies.data["ENERG_HI"]
+            spectra = f["SPECTRA"]
+
+            if f[0].header["MODLUNIT"] == "photons/cm^2/s":
+                log.info(
+                    "This table model has flux units of photons/cm^2/s. It will be converted to differential Flux by dividing by the energy bin widths. Logarithmic centers will be turned off. "
+                )
+                delta_ene = ene_hi - ene_lo
+                self._spectrum = spectra.data["INTPSPEC"] / delta_ene
+                self._log_centers = False
+            else:
+                self._spectrum = spectra.data["INTPSPEC"]
 
             # log centers
 
@@ -891,10 +935,6 @@ class XSPECTableModel(object):
 
             self._n_params = len(self._names)
 
-            spectra = f["SPECTRA"]
-
-            self._spectrum = spectra.data["INTPSPEC"]
-
             self._params_dict = {}
 
             for i, name in enumerate(self._names):
@@ -907,9 +947,10 @@ class XSPECTableModel(object):
 
                     try:
 
-                        this_dict["values"] = spectra.data["PARAMVAL{%d}" % i]
+                        this_dict["values"] = spectra.data[f"PARAMVAL{i}"]
+
+                    except KeyError:
 
-                    except:
                         this_dict["values"] = spectra.data["PARAMVAL"][:, i]
 
                 else:
@@ -954,7 +995,6 @@ class XSPECTableModel(object):
         tmf.save_data(overwrite=overwrite)
 
 
-
 def convert_old_table_model(model_name: str):
     from pandas import HDFStore
 
@@ -963,19 +1003,20 @@ def convert_old_table_model(model_name: str):
     data_dir_path: Path = get_user_data_path()
 
     # Sanitize the data file
-    
+
     filename_sanitized = data_dir_path / f"{model_name}.h5"
-    
-    
+
     if not filename_sanitized.exists():
 
-        log.error("The data file %s does not exists. Did you use the "
-             "TemplateFactory?" % (filename_sanitized))
-         
-        raise MissingDataFile( )
-    
+        log.error(
+            "The data file %s does not exists. Did you use the "
+            "TemplateFactory?" % (filename_sanitized)
+        )
+
+        raise MissingDataFile()
+
     with HDFStore(filename_sanitized) as store:
-    
+
         data_frame = store["data_frame"]
 
         parameters_grids = collections.OrderedDict()
@@ -985,29 +1026,28 @@ def convert_old_table_model(model_name: str):
         for key in list(store.keys()):
 
             match = re.search("p_([0-9]+)_(.+)", key)
-            
+
             if match is None:
-                
+
                 continue
-            
+
             else:
-                
+
                 tokens = match.groups()
-                
+
                 this_parameter_number = int(tokens[0])
                 this_parameter_name = str(tokens[1])
-                
-                
+
                 if not this_parameter_number == processed_parameters:
 
                     log.error("Parameters out of order!")
 
                     raise AssertionError()
-                
+
                 parameters_grids[this_parameter_name] = store[key]
-                
+
                 processed_parameters += 1
-                
+
         energies = np.array(store["energies"])
 
         shape = [len(x) for _, x in parameters_grids.items()]
@@ -1020,30 +1060,26 @@ def convert_old_table_model(model_name: str):
 
             data = data_frame[e].values.reshape(*shape[:-1])
 
-            grid[...,i] = data
-        
-                     
-        
+            grid[..., i] = data
+
         # Now get the metadata
-            
+
         metadata = store.get_storer("data_frame").attrs.metadata
-        
+
         description = metadata["description"]
-        
+
         name = metadata["name"]
-        
+
         interpolation_degree = metadata["interpolation_degree"]
-        
+
         spline_smoothing_factor = metadata["spline_smoothing_factor"]
-        
+
         # Make the dictionary of parameters
 
         function_definition = collections.OrderedDict()
 
         function_definition["description"] = description
 
-
-
     template_file: TemplateFile = TemplateFile(
         name=name,
         description=description,
@@ -1052,9 +1088,7 @@ def convert_old_table_model(model_name: str):
         grid=grid,
         energies=energies,
         parameters=parameters_grids,
-        parameter_order=list(parameters_grids.keys()))
+        parameter_order=list(parameters_grids.keys()),
+    )
 
     template_file.save(filename_sanitized)
-
-
-        
