@@ -1,5 +1,6 @@
 from __future__ import print_function
-__author__ = 'giacomov'
+
+__author__ = "giacomov"
 
 # Import IPython display facility, if available. Otherwise,
 # create a wrapper which just uses print
@@ -16,6 +17,7 @@ except ImportError:
         """
         print(args)
 
+
 try:
 
     from IPython.display import Latex
