@@ -1,16 +1,17 @@
 # If threeml is not installed, we create our own log,
 # otherwise, just print to the 3ML one!
 
-
 import logging
 import logging.handlers as handlers
+from contextlib import contextmanager
 from pathlib import Path
 
-from astromodels.utils.configuration import astromodels_config
 from rich.console import Console
 from rich.logging import RichHandler
 from rich.theme import Theme
 
+from astromodels.utils.configuration import astromodels_config
+
 from .file_utils import _get_data_file_path
 
 try:
@@ -19,11 +20,24 @@ try:
 
     has_threeml = True
 
-except:
+except ImportError:
 
     has_threeml = False
 
 
+DEBUG_NODE_LEVEL = 9
+logging.addLevelName(DEBUG_NODE_LEVEL, "DEBUG_NODE")
+
+
+def debug_node(self, message, *args, **kws):
+    if self.isEnabledFor(DEBUG_NODE_LEVEL):
+        # Yes, logger takes its '*args' as 'args'.
+        self._log(DEBUG_NODE_LEVEL, message, args, **kws)
+
+
+logging.Logger.debug_node = debug_node
+
+
 def get_path_of_log_dir():
 
     # we use the 3ML log path to simplify things
@@ -52,7 +66,9 @@ def get_path_of_log_file(log_file: str) -> Path:
     """
     returns the path of the log files
     """
-    assert log_file in _log_file_names, f"{log_file} is not one of {_log_file_names}"
+    assert (
+        log_file in _log_file_names
+    ), f"{log_file} is not one of {_log_file_names}"
 
     return get_path_of_log_dir() / log_file
 
@@ -102,9 +118,6 @@ _console_formatter = logging.Formatter(
 )
 
 
-
-
-
 _theme = {}
 
 # Banner
@@ -123,10 +136,10 @@ _theme["logging.level.debug"] = f"{astromodels_config.logging.debug_style}"
 _theme["logging.level.error"] = f"{astromodels_config.logging.error_style}"
 _theme["logging.level.info"] = f"{astromodels_config.logging.info_style}"
 _theme["logging.level.warning"] = f"{astromodels_config.logging.warn_style}"
+_theme["logging.level.degub_node"] = "light_goldenrod1"
 
 
-
-#mytheme = Theme().read(_get_data_file_path("log_theme.ini"))
+# mytheme = Theme().read(_get_data_file_path("log_theme.ini"))
 mytheme = Theme(_theme)
 console = Console(theme=mytheme)
 
@@ -140,6 +153,26 @@ astromodels_console_log_handler.setLevel("INFO")
 warning_filter = LogFilter(logging.WARNING)
 
 
+@contextmanager
+def silence_console_log():
+    """
+    temporarily silence the console and progress bars
+    """
+    current_console_logging_level = astromodels_console_log_handler.level
+    current_usr_logging_level = astromodels_usr_log_handler.level
+
+    astromodels_console_log_handler.setLevel(logging.CRITICAL)
+    astromodels_usr_log_handler.setLevel(logging.CRITICAL)
+
+    try:
+        yield
+
+    finally:
+
+        astromodels_console_log_handler.setLevel(current_console_logging_level)
+        astromodels_usr_log_handler.setLevel(current_usr_logging_level)
+
+
 def silence_warnings():
     """
     supress warning messages in console and file usr logs
@@ -170,7 +203,7 @@ def setup_logger(name):
     log = logging.getLogger(name)
 
     # this must be set to allow debug messages through
-    log.setLevel(logging.DEBUG)
+    log.setLevel(DEBUG_NODE_LEVEL)
 
     # add the handlers
 
